package com.ultigesture.ugtest;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Handler;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.ultigesture.ug.UGDevice;
import com.ultigesture.ug.UGManager;
import com.ultigesture.ug.UGProfile;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;


public class MainActivity extends AppCompatActivity {

    private TextView mTextView1, mTextView2, mTextView3, mTextView4, mTextView5, mTextView6;
    private Button startButton, stopButton, calibrationButton;
    private Timer mTimer;
    private Handler mHandler = new Handler();

    public static final int PERMISSION_REQUEST_BLE = 100;
    private static final String[] BLE_PERMISSIONS = {
            Manifest.permission.BLUETOOTH,
            Manifest.permission.BLUETOOTH_ADMIN,
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION
    };

    private int samplingInterval = 100;
    private ArrayList<UGDevice> unconnectedUGDevices = new ArrayList<>();
    private ArrayList<UGDevice> connectedUGDevices = new ArrayList<>();
    private UGManager mUGManager;

    private Handler doActionHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            int msgId = msg.what;
            switch (msgId) {
                case 1:
                    if (unconnectedUGDevices.size() != 0){
                        unconnectedUGDevices.get(0).connect(mConnectionStateChangeCallback);
                    }
                    break;

                default:
                    break;
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        ui_init();
        ble_init();

        mTimer = new Timer();
        setTimerTask();

        mUGManager = new UGManager(this);
        mUGManager.startScan(mScanCallback);
    }



    private void ui_init(){

        mTextView1 = (TextView) findViewById(R.id.textView1);
        mTextView2 = (TextView) findViewById(R.id.textView2);
        mTextView3 = (TextView) findViewById(R.id.textView3);
        mTextView4 = (TextView) findViewById(R.id.textView4);
        mTextView5 = (TextView) findViewById(R.id.textView5);
        mTextView6 = (TextView) findViewById(R.id.textView6);
        startButton = (Button) findViewById(R.id.startButton);
        stopButton = (Button) findViewById(R.id.stopButton);
        calibrationButton = (Button) findViewById(R.id.calibrationButton);

        startButton.setOnClickListener(new View.OnClickListener(){

            @Override
            public void onClick(View view) {
                for (int i = 0; i < connectedUGDevices.size(); i++){
                    connectedUGDevices.get(i).startDataSensing(mDataAvailableCallback, samplingInterval);
                }
            }
        });

        stopButton.setOnClickListener(new View.OnClickListener(){

            @Override
            public void onClick(View view) {
                for (int i = 0; i < connectedUGDevices.size(); i++){
                    connectedUGDevices.get(i).stopDataSensing();
                }
            }
        });

        calibrationButton.setOnClickListener(new View.OnClickListener(){

            @Override
            public void onClick(View view) {
                for (int i = 0; i < connectedUGDevices.size(); i++){
                    connectedUGDevices.get(i).calibrateSensor();
                }
            }
        });
    }

    private void ble_init() {
        while(!hasPermissionGranted(BLE_PERMISSIONS) ) {
            requestBLEPermissions();
        }

        BluetoothManager mBluetoothManager = (BluetoothManager) this.getSystemService(Context.BLUETOOTH_SERVICE);
        BluetoothAdapter mBluetoothAdapter = mBluetoothManager.getAdapter();
        if (mBluetoothAdapter == null || !mBluetoothAdapter.isEnabled()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            this.startActivityForResult(enableBtIntent, 1);
        }

    }

    private void setTimerTask() {
        mTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                Message message = new Message();
                message.what = 1;
                doActionHandler.sendMessage(message);
            }
        }, 0, 100);
    }



    private UGManager.ScanCallback mScanCallback = new UGManager.ScanCallback(){

        @Override
        public void onScanCallback(UGDevice device) {
            System.out.println("received devcie: name: " + device.getName() + "  mac: " + device.getAddress());
            boolean flag = false;
            for (int i = 0; i < unconnectedUGDevices.size(); i++){
                if (unconnectedUGDevices.get(i).getAddress().equals(device.getAddress())){
                    flag = true;
                }
            }
            if (!flag) {
                unconnectedUGDevices.add(device);
            }
        }
    };



    private UGDevice.StatusChangeCallback mConnectionStateChangeCallback = new UGDevice.StatusChangeCallback(){
        @Override
        public void onStatusChange(UGDevice device, int status) {
            System.out.println("onConnectionStateChange device: " + device.getAddress() + "status: " + status);
            if (status == UGProfile.STATUS_CONNECTED){
                for (int i = 0; i < unconnectedUGDevices.size(); i++){
                    if (unconnectedUGDevices.get(i).getAddress().equals(device.getAddress())){
                        unconnectedUGDevices.remove(i);
                    }
                }
                connectedUGDevices.add(device);
                device.startDataSensing(mDataAvailableCallback, samplingInterval);
            } else if (status == UGProfile.STATUS_DISCONNECTED){
                for (int i = 0; i < connectedUGDevices.size(); i++){
                    if (connectedUGDevices.get(i).getAddress().equals(device.getAddress())){
                        connectedUGDevices.remove(i);
                    }
                }
            }
        }
    };


    private UGDevice.DataAvailableCallback mDataAvailableCallback = new UGDevice.DataAvailableCallback(){

        @Override
        public void onSensorDataAvailable(UGDevice ugDevice, float[] data) {
            float AccX = data[0];
            float AccY = data[1];
            float AccZ = data[2];
            float GyroX = data[3];
            float GyroY = data[4];
            float GyroZ = data[5];
            float MagX = data[6];
            float MagY = data[7];
            float MagZ = data[8];

            final String text = "UGDevice: " + ugDevice.getAddress() + " (m/s^2   dps   gauss)"
                    + "\nAccX: " + format(AccX) + " AccY: " + format(AccY) + " AccZ: " + format(AccZ)
                    + "\nGyroX: " + format(GyroX) + " GyroY: " + format(GyroY) + " GyroZ: " + format(GyroZ)
                    + "\nMagX: " + format(MagX) + " MagY: " + format(MagY) + " MagZ: " + format(MagZ);
            System.out.println(text);

            for (int i = 0; i < connectedUGDevices.size(); i++){
                if (ugDevice.getAddress().equals(connectedUGDevices.get(i).getAddress())){
                    final int index = i;
                    mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            switch (index){
                                case 0:
                                    mTextView1.setText(text);
                                    break;

                                case 1:
                                    mTextView2.setText(text);
                                    break;

                                case 2:
                                    mTextView3.setText(text);
                                    break;

                                case 3:
                                    mTextView4.setText(text);
                                    break;

                                case 4:
                                    mTextView5.setText(text);
                                    break;

                                case 5:
                                    mTextView6.setText(text);
                                    break;

                                default:
                                    break;
                            }
                        }
                    });
                    break;
                }
            }
        }

        @Override
        public void onBatteryLvlAvailable(UGDevice device, int data) {
            System.out.println("battery level: " + data);
        }
    };


    private void requestBLEPermissions() {
        if (shouldShowRequestPermissionRationale(BLE_PERMISSIONS)) {
            Toast.makeText(this, "BLE", Toast.LENGTH_SHORT).show();
        } else {
            requestPermissions(BLE_PERMISSIONS, PERMISSION_REQUEST_BLE);
        }
    }

    private boolean hasPermissionGranted(String[] permissions) {
        for(String permission : permissions) {
            if(checkSelfPermission(permission) != PackageManager.PERMISSION_GRANTED) {
                System.out.println("permission denied?");
                return false;
            }
        }
        return true;
    }

    private boolean shouldShowRequestPermissionRationale(String[] permissions) {
        for (String permission : permissions) {
            if (shouldShowRequestPermissionRationale(permission)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions,
                                           int[] grantResults) {

        if(requestCode == PERMISSION_REQUEST_BLE ) {
            if (grantResults.length == BLE_PERMISSIONS.length) {
                for (int result : grantResults) {
                    if (result != PackageManager.PERMISSION_GRANTED) {
                        break;
                    }
                }
            } else {

            }
        } else {

        }
    }


    public String format(float value) {
        BigDecimal b = new BigDecimal(value);
        float floatValue  =   b.setScale(2, BigDecimal.ROUND_HALF_UP).floatValue();
        return String.valueOf(floatValue);
    }


    @Override
    protected void onStop() {
        super.onStop();
        mUGManager.close();
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        mUGManager.close();
    }
}
